"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var baseplatform_1 = require("../baseplatform");
var emu_1 = require("../emu");
var audio_1 = require("../audio");
var VICDUAL_PRESETS = [
    { id: 'minimal.c', name: 'Minimal Example' },
    { id: 'hello.c', name: 'Hello World' },
    { id: 'gfxtest.c', name: 'Graphics Test' },
    { id: 'soundtest.c', name: 'Sound Test' },
    { id: 'snake1.c', name: 'Siege Game (Prototype)' },
    { id: 'snake2.c', name: 'Siege Game (Full)' },
    { id: 'music.c', name: 'Music Player' },
];
var VicDualDisplay = /** @class */ (function () {
    function VicDualDisplay() {
        this.palbank = 0;
        this.palette = [
            0xff000000,
            0xff0000ff,
            0xff00ff00,
            0xff00ffff,
            0xffff0000,
            0xffff00ff,
            0xffffff00,
            0xffffffff // white
        ];
        // default PROM
        this.colorprom = [
            0xe0, 0x60, 0x20, 0x60, 0xc0, 0x60, 0x40, 0xc0,
            0x20, 0x40, 0x60, 0x80, 0xa0, 0xc0, 0xe0, 0x0e,
            0xe0, 0xe0, 0xe0, 0xe0, 0x60, 0x60, 0x60, 0x60,
            0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0,
        ];
    }
    // videoram 0xc000-0xc3ff
    // RAM      0xc400-0xc7ff
    // charram  0xc800-0xcfff
    VicDualDisplay.prototype.drawScanline = function (ram, pixels, sl) {
        if (sl >= 224)
            return;
        var pixofs = sl * 256;
        var outi = pixofs; // starting output pixel in frame buffer
        var vramofs = (sl >> 3) << 5; // offset in VRAM
        var yy = sl & 7; // y offset within tile
        for (var xx = 0; xx < 32; xx++) {
            var code = ram[vramofs + xx];
            var data = ram[0x800 + (code << 3) + yy];
            var col = (code >> 5) + (this.palbank << 3);
            var color1 = this.palette[(this.colorprom[col] >> 1) & 7];
            var color2 = this.palette[(this.colorprom[col] >> 5) & 7];
            for (var i = 0; i < 8; i++) {
                var bm = 128 >> i;
                pixels[outi] = (data & bm) ? color2 : color1;
                /* TODO
                if (framestats) {
                  framestats.layers.tiles[outi] = (data&bm) ? colorprom[col+8] : colorprom[col];
                }
                */
                outi++;
            }
        }
    };
    return VicDualDisplay;
}());
var CARNIVAL_KEYCODE_MAP = emu_1.makeKeycodeMap([
    [emu_1.Keys.A, 2, -0x20],
    [emu_1.Keys.B, 2, -0x40],
    [emu_1.Keys.LEFT, 1, -0x10],
    [emu_1.Keys.RIGHT, 1, -0x20],
    [emu_1.Keys.UP, 1, -0x40],
    [emu_1.Keys.DOWN, 1, -0x80],
    [emu_1.Keys.START, 2, -0x10],
    [emu_1.Keys.P2_START, 3, -0x20],
    [emu_1.Keys.SELECT, 3, 0x8],
]);
var XTAL = 15468000.0;
var scanlinesPerFrame = 0x106;
var vblankStart = 0xe0;
var vsyncStart = 0xec;
var vsyncEnd = 0xf0;
var cpuFrequency = XTAL / 8;
var hsyncFrequency = XTAL / 3 / scanlinesPerFrame;
var vsyncFrequency = hsyncFrequency / 0x148;
var cpuCyclesPerLine = cpuFrequency / hsyncFrequency;
var timerFrequency = 500; // input 2 bit 0x8
var cyclesPerTimerTick = cpuFrequency / (2 * timerFrequency);
var VicDualPlatform = /** @class */ (function (_super) {
    __extends(VicDualPlatform, _super);
    function VicDualPlatform() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.reset_disable = false;
        _this.cpuFrequency = XTAL / 8; // MHz
        _this.canvasWidth = 256;
        _this.numTotalScanlines = 262;
        _this.numVisibleScanlines = 224;
        _this.defaultROMSize = 0x4040;
        return _this;
    }
    VicDualPlatform.prototype.getPresets = function () { return VICDUAL_PRESETS; };
    VicDualPlatform.prototype.getKeyboardMap = function () { return CARNIVAL_KEYCODE_MAP; };
    VicDualPlatform.prototype.getKeyboardFunction = function () {
        var _this = this;
        return function (o) {
            // reset when coin inserted
            if (o.index == 3 && o.mask == 0x8 && !_this.reset_disable) {
                _this.cpu.reset();
                console.log("coin inserted");
                console.log(_this.inputs);
            }
            // don't allow repeated resets in short period of time
            _this.reset_disable = true;
            clearTimeout(_this.reset_disable_timer);
            _this.reset_disable_timer = setTimeout(function () { _this.reset_disable = false; }, 1100);
        };
    };
    ;
    VicDualPlatform.prototype.getVideoOptions = function () { return { rotate: -90 }; };
    VicDualPlatform.prototype.newRAM = function () {
        return new Uint8Array(0x1000);
    };
    VicDualPlatform.prototype.newMembus = function () {
        var _this = this;
        return {
            read: emu_1.newAddressDecoder([
                [0x0000, 0x7fff, 0x3fff, function (a) { return _this.rom ? _this.rom[a] : null; }],
                [0x8000, 0xffff, 0x0fff, function (a) { return _this.ram[a]; }],
            ]),
            write: emu_1.newAddressDecoder([
                [0x8000, 0xffff, 0x0fff, function (a, v) { _this.ram[a] = v; }],
            ]),
        };
    };
    VicDualPlatform.prototype.newIOBus = function () {
        var _this = this;
        return {
            read: function (addr) {
                return _this.inputs[addr & 3];
            },
            write: function (addr, val) {
                if (addr & 0x1) {
                    _this.psg.selectRegister(val & 0xf);
                }
                ; // audio 1
                if (addr & 0x2) {
                    _this.psg.setData(val);
                }
                ; // audio 2
                if (addr & 0x8) { }
                ; // TODO: assert coin status
                if (addr & 0x40) {
                    _this.display.palbank = val & 3;
                }
                ; // palette
            }
        };
    };
    VicDualPlatform.prototype.start = function () {
        _super.prototype.start.call(this);
        this.inputs.set([0xff, 0xff, 0xff, 0xff ^ 0x8]); // most things active low
        this.display = new VicDualDisplay();
        this.audio = new audio_1.MasterAudio();
        this.psg = new audio_1.AY38910_Audio(this.audio);
    };
    VicDualPlatform.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.psg.reset();
    };
    VicDualPlatform.prototype.startScanline = function (sl) {
        this.inputs[2] &= ~0x8;
        this.inputs[2] |= ((this.cpu.getTstates() / cyclesPerTimerTick) & 1) << 3;
        if (sl == vblankStart)
            this.inputs[1] |= 0x8;
        if (sl == vsyncEnd)
            this.inputs[1] &= ~0x8;
    };
    VicDualPlatform.prototype.drawScanline = function (sl) {
        this.display.drawScanline(this.ram, this.video.getFrameData(), sl);
    };
    VicDualPlatform.prototype.loadROM = function (title, data) {
        _super.prototype.loadROM.call(this, title, data);
        if (data.length >= 0x4020 && (data[0x4000] || data[0x401f])) {
            this.display.colorprom = data.slice(0x4000, 0x4020);
        }
    };
    VicDualPlatform.prototype.loadState = function (state) {
        _super.prototype.loadState.call(this, state);
        this.display.palbank = state.pb;
    };
    VicDualPlatform.prototype.saveState = function () {
        var state = _super.prototype.saveState.call(this);
        state['pb'] = this.display.palbank;
        return state;
    };
    return VicDualPlatform;
}(baseplatform_1.BasicZ80ScanlinePlatform));
emu_1.PLATFORMS['vicdual'] = VicDualPlatform;
//# sourceMappingURL=vicdual.js.map